
//#include "hime.h"
#include "lneut.h"

#include "TFile.h"
#include "TDirectory.h"
#include "TTree.h"
#include "TCanvas.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TGraph2DErrors.h"
#include "TVector3.h"

#include <iostream>
#include <string>
#include <cmath>

#include "CKinema.hh"
#include "CParticle.hh"
#include "CParticleDefinition.hh"
#include "CAnalysis.hh"

#include "Analysis_sub.C"

//memory leakのerror処理はgdbを走らせれば分かるのでやってない。

void Analysis(std::string filename, std::string method, Long64_t nentries)
{
  TDirectory* topdir = gDirectory;

  TCanvas* canvas = new TCanvas("canvas","",200,10,900,900);

  new TH1D("rms", "title", 100, 0., 20.);
  new TH1D("theta_exp", "title", 100, 0., 3.14159);
  new TH1D("theta_cm_exp", "title", 100, 0., 3.14159);

  TH2D* hthetaenergy_lab
    = new TH2D("thetaenergy_lab", "title", 100, 0., 3.14159, 100, 0., ein+20);
  TH2D* hthetaenergy_lab_gate
    = new TH2D("thetaenergy_lab_gate", "title", 100, 0., 3.14159, 100, 0., ein+20);
  TH2D* hthetaenergy_cm
    = new TH2D("thetaenergy_cm", "title", 100, 0., 3.14159, 100, 0., ein+20);
  TH2D* hthetaenergy_cm_gate
    = new TH2D("thetaenergy_cm_gate", "title", 100, 0., 3.14159, 100, 0., ein+20);

  TH1D* htheta_cm = new TH1D("theta_cm", "title", 100, 0., 3.14159);
  TH1D* htheta_cm_gate = new TH1D("theta_cm_gate", "title", 100, 0., 3.14159);

  TH1D* hpx = new TH1D("n_px", "title", 100, 0.-2, 0.+2);
  TH1D* hpy = new TH1D("n_py", "title", 100, 0.-2, 0.+2);
  TH1D* hpz = new TH1D("n_pz", "title", 100, 0.-2, 0.+2);
  TH1D* hcpx = new TH1D("cp_px", "title", 100, 0.-2, 0.+2);
  TH1D* hcpy = new TH1D("cp_py", "title", 100, 0.-2, 0.+2);
  TH1D* hcpz = new TH1D("cp_pz", "title", 100, 0.-2, 0.+2);
  TH1D* herel = new TH1D("Erel", "title", 100, 0., 1.);

  {
    TFile* file = TFile::Open(filename.c_str(), "READ");
    if(!file) return;

    TTree* tree;
    file->GetObject("Koyomi",tree);

    TFile* workfile = TFile::Open("root/temp.root","recreate");
    if(!workfile) return;

    TTree* newtree = new TTree("Hitagi", "title");

    CDataHimeSim* datasim = 0;
    TBranch *bdatasim = 0;
    if(tree->SetBranchAddress("hime",&datasim,&bdatasim) == TTree::kMissingBranch) return;

    CDataHime* dataexp = new CDataHime;
    newtree->Branch("hime", &dataexp);

    topdir->cd();

    Long64_t count = 0;
    if(nentries < 0) nentries = tree->GetEntriesFast();
    for(int i=0; i<nentries; i++) {
      ++count;
      if(count%10000 == 0) std::cout << "\r converted events:" << count << std::flush;

      Long64_t entry = tree->LoadTree(i);
      bdatasim->GetEntry(entry);

      if(!datasim->fEventFlag) continue;

      dataexp->Reset();
      SimToExp(datasim, dataexp, method);

      newtree->Fill();
    }
    std::cout << "\r converted events:" << count << std::endl;

    tree->ResetBranchAddresses();
    
    workfile->Write();
    workfile->Close();
    file->Close();
  }

  {
    CParticle proton = CParticleDefinition::GetParticle("proton");
    CParticle neutron = CParticleDefinition::GetParticle("neutron");
    CParticle chargedparticle = CParticleDefinition::GetParticle("9Li");

    TFile* filesim = TFile::Open(filename.c_str(), "READ");
    if(!filesim) return;

    TTree* treesim;
    filesim->GetObject("Koyomi",treesim);

    CDataHimeSim* datasim = 0;
    TBranch *bdatasim = 0;
    if(treesim->SetBranchAddress("hime",&datasim,&bdatasim) == TTree::kMissingBranch) return;

    TFile* fileexp = TFile::Open("root/temp.root","READ");
    if(!fileexp) return;

    TTree* treeexp;
    fileexp->GetObject("Hitagi",treeexp);

    CDataHime* dataexp = 0;
    TBranch *bdataexp = 0;
    if(treeexp->SetBranchAddress("hime",&dataexp,&bdataexp) == TTree::kMissingBranch) return;

    Long64_t count = 0;
    if(nentries < 0) nentries = treesim->GetEntriesFast();
    for(Long64_t i=0, j=0; i<nentries; i++) {
      Long64_t entrysim = treesim->LoadTree(i);
      bdatasim->GetEntry(entrysim);

      if(!datasim->fEventFlag) continue;

      Long64_t entryexp = treeexp->LoadTree(j++);
      bdataexp->GetEntry(entryexp);

      //protonの真の角度分布をグラフ化してそれと比較する
      Double_t theta_lab, energy_lab;
      Double_t theta_cm, energy_cm;
      if(GetFirstProtonInformation(datasim, proton) != 0){
	theta_lab = proton.theta();
	energy_lab = proton.k();

	neutron.p(datasim->fMomentum.x(),datasim->fMomentum.y(),datasim->fMomentum.z());
	Double_t beta = neutron.p()/(neutron.e()+proton.m());
	CKinema::LorentzBoost(proton, beta, 2);

	theta_cm = proton.theta();
	energy_cm = proton.k();

	htheta_cm->Fill(theta_cm);
	hthetaenergy_lab->Fill(theta_lab, energy_lab);
	hthetaenergy_cm->Fill(theta_cm, energy_cm);
	if(dataexp->fEventFlag){
	  htheta_cm_gate->Fill(theta_cm);
	  hthetaenergy_lab_gate->Fill(theta_lab, energy_lab);
	  hthetaenergy_cm_gate->Fill(theta_cm, energy_cm);
	}
      }

      if(dataexp->fEventFlag){
	++count;
	if(count%100 == 0) std::cout << "\r analysed events:" << count << std::flush;
      }else{
	continue;
      }

      TVector3 np = datasim->fMomentum;
      TVector3 cpp = datasim->fMomentumChargedParticle;

      //fold momentum
      neutron.p(dataexp->fP.Px(), dataexp->fP.Py(), dataexp->fP.Pz());
      FoldChargedParticle(cpp, chargedparticle, dele, estrag, multi_theta);

      double erel = CKinema::InvariantMass(neutron, chargedparticle)
	- neutron.m() - chargedparticle.m();
      herel->Fill(erel);

      hpx->Fill(100*(neutron.p(0)-np.Px())/np.Mag());
      hpy->Fill(100*(neutron.p(1)-np.Py())/np.Mag());
      hpz->Fill(100*(neutron.p(2)-np.Pz())/np.Mag());

      hcpx->Fill(100*(chargedparticle.p(0)-cpp.Px())/cpp.Mag());
      hcpy->Fill(100*(chargedparticle.p(1)-cpp.Py())/cpp.Mag());
      hcpz->Fill(100*(chargedparticle.p(2)-cpp.Pz())/cpp.Mag());
    }
    std::cout << "\r analysed events:" << count << std::endl;

    treeexp->ResetBranchAddresses();
    treesim->ResetBranchAddresses();
    
    fileexp->Close();
    filesim->Close();
    topdir->cd();
  }

  {
    canvas->Divide(3,3);
    canvas->cd(1);
    hpx->Draw();
    hpx->Fit("gaus");
    canvas->cd(2);
    hpy->Draw();
    hpy->Fit("gaus");
    canvas->cd(3);
    hpz->Draw();
    hpz->Fit("gaus");
    canvas->cd(4);
    hcpx->Draw();
    hcpx->Fit("gaus");
    canvas->cd(5);
    hcpy->Draw();
    hcpy->Fit("gaus");
    canvas->cd(6);
    hcpz->Draw();
    hcpz->Fit("gaus");
    canvas->cd(7);
    herel->Draw();

    canvas->cd(8);
    hthetaenergy_lab->Draw("COLZBARBOX");
    canvas->cd(9);
    hthetaenergy_cm->Draw("COLZBARBOX");

    canvas->Update();
  }
}

